import { HTMLAttributes, PropsWithChildren, AllHTMLAttributes, ForwardRefExoticComponent, PropsWithoutRef, RefAttributes, ElementType } from 'react';

declare type ComponentAttributes = Record<any, any>;
declare type CustomAttributes = {
    [k: string]: CustomAttributeValue | undefined;
};
declare type CustomAttributeValue = string | number | boolean;
/**
 * A Special type that does not allow on @S any key from @T
 *
 * @export
 * @typedef {ExcludeFromHTMLElement}
 * @template S
 * @template T
 */
declare type ExcludeFromHTMLElement<S, T> = S & {
    [K in keyof T]?: never;
};
/**
 * A class definition can be any of the following:
 * - A string with a single word: 'btn'
 * - A multiple word string: 'btn btn-primary'
 * - An array with more ClassDefinition[]
 * - A {key: ClassDefinition} object, where `key` will be used as a prefix
 * - A function that returns a ClassDefinition: () => ['a','b']
 *
 * Any ClassDefinition will be recursively parsed until it is just a string.
 *
 * @example: {
 *   a: 'btn',
 *   b: 'btn btn-primary',
 *   c: ['a', 'b', 'c'],
 *   d: ['a', { hover: 'b' }, ['c', 'd'], () => ({ dark: ['x', 'y', 'z'] })]
 * }
 *
 * @export
 * @typedef {ClassDefinition}
 */
declare type ClassDefinition = string | ClassDefinitionFn | {
    [key: string]: ClassDefinition;
} | ClassDefinition[];
declare type ClassDefinitionFn = ((value: CustomAttributeValue) => ClassDefinition);
declare type ComposerConfigOptions<T> = {
    [K in T as string]: ClassDefinition;
};
declare type OptionAlias<T> = {
    [key: string]: keyof T;
};
declare type ClassMixerFactory<T> = (when: string[], arg: T) => ClassMixer;
declare type ClassMixerFn = (css: Set<string>) => void;
declare type ClassMixer = {
    when: string[];
    run: ClassMixerFn;
};
interface ComposerConfig<P extends ComponentAttributes, A = {}> {
    /**
     * base classes, will always be applied
     *
     * @type {ClassDefinition}
     * @see {ClassDefinition}
     */
    base: ClassDefinition;
    /**
     * options are [key,value] pairs, where the key is the prop name.
     * key can not be present in the native HTML element props
     * to use a native prop, prefix it with `$` like `{ $disabled: 'btn-disabled' }`
     *
     * @type {ExcludeFromHTMLElement<ComposerConfigOptions<P>, A>}
     */
    options: ExcludeFromHTMLElement<ComposerConfigOptions<P>, A>;
    /**
     * A mixer is an object that looks like {when: [], run: () => {}}
     * and runs the function when the props match what was defined in the in the array
     *
     * Conditions can be specific like when: ['disabled.true'] or generic like ['title.*']
     *
     * @type {?ClassMixer[]}
     */
    mix?: ClassMixer[];
    /**
     * a simple {key: value} object, where value mult be a key of {options}
     *
     * @type {?OptionAlias<P>}
     */
    alias?: OptionAlias<P>;
}
/**
 * Returns the classnames parsed from the options object and a set of properties to forward
 *
 * @export
 * @interface ClassComposerReturns
 * @typedef {ClassComposerReturns}
 * @template A extends HTMLAttributes<any>
 */
interface ClassComposerReturns<A extends HTMLAttributes<any>> {
    /**
     * a string with css classnames
     */
    className: string;
    /**
     * Contains any prop that is native to the HTML Element being forwarded
     *
     * @type {PropsWithChildren<A>}
     */
    forwardProps: PropsWithChildren<A>;
}
/**
 * A component that was created using createComponent, will be an instance of EL and  allow any property present in P and A
 *
 * @export
 * @typedef {ComposedComponent}
 * @template P extends CustomAttributes
 * @template EL extends HTMLElement
 * @template A extends HTMLAttributes<EL> = AllHTMLAttributes<EL>
 */
declare type ComposedComponent<P extends CustomAttributes, EL extends HTMLElement, A extends HTMLAttributes<EL> = AllHTMLAttributes<EL>> = ForwardRefExoticComponent<PropsWithoutRef<P & A> & RefAttributes<EL>>;

/**
 * Create a React component that extends and creates a forwardRef of native HTML element.
 *
 *
 * @template P extends CustomAttributes
 * @template EL extends HTMLElement
 * @template A extends HTMLAttributes<EL> = AllHTMLAttributes<EL>
 * @param {ElementType} el  the HTML Element to extend, usually a JSX tagname from {React.IntrinsicElements}
 * @param {ComposerConfig<P, A>} config  the Composer Config object/
 * @param {Partial<P>} [defaults={}]  any defaults to be applied to the parameters.
 * @returns {ComposedComponent<P, EL, A>}
 *
 * @see {ComposerConfig}
 * @see {ClassDefinition}
 */
declare function createComponent<P extends CustomAttributes, EL extends HTMLElement, A extends HTMLAttributes<EL> = AllHTMLAttributes<EL>>(el: ElementType, config: ComposerConfig<P, A>, defaults?: Partial<P>): ComposedComponent<P, EL, A>;

declare function parseDefinition(propValue: CustomAttributeValue, propDefinition: ClassDefinition, prefix?: string): string[];

declare const shouldMix: (when: string[], optionArr: string[]) => boolean;
declare const mixFunction: ClassMixerFactory<ClassMixerFn>;
declare const mixAddClass: ClassMixerFactory<ClassDefinition>;
declare const mixRemoveClass: ClassMixerFactory<ClassDefinition>;

/**
 * Hook that parses a set of parameters based on a configuration object and returns applicable classnames
 *
 *
 * @export
 * @template P extends ComponentAttributes
 * @template A extends HTMLAttributes<any> = HTMLAttributes<any>
 * @param {ComposerConfig<P, A>} config the configuration object
 * @param {(P & A)} props the props to parse
 * @returns {ClassComposerReturns<A>} { className, forwardProps }
 */
declare function useClassComposer<P extends ComponentAttributes, A extends HTMLAttributes<any> = HTMLAttributes<any>>(config: ComposerConfig<P, A>, props: P & A): ClassComposerReturns<A>;

export { ClassComposerReturns, ClassDefinition, ClassDefinitionFn, ClassMixer, ClassMixerFactory, ClassMixerFn, ComponentAttributes, ComposedComponent, ComposerConfig, ComposerConfigOptions, CustomAttributeValue, CustomAttributes, ExcludeFromHTMLElement, OptionAlias, createComponent, mixAddClass, mixFunction, mixRemoveClass, parseDefinition, shouldMix, useClassComposer };
